!c Description:
!c   ١ϡ, Ǯϳشؿ G ʿջƳ롼
!c
!c Current Code Owner: 
!c   sugiyama@gfd-dennou.org
!c
!c Histry: 
!c   Version    Date          Comment
!c   -------    ----------    --------
!c   1.0        2003-02-25    
!c
!c Copyright (C) SUGIYAMA Ko-ichiro, 2003, All rights reserved

module interface_mingibbs
  interface
     
     recursive subroutine mingibbs(temp, pres, emol, struct, property, mol_nxt, mol_ini)
       use prm_type, only: type_property
       real(8), intent(in)               :: temp  
       real(8), intent(in)               :: pres
       real(8), intent(in)               :: emol(:)
       real(8), intent(in)               :: struct(:,:)
       real(8), intent(in), optional     :: mol_ini(:)
       real(8), intent(out)              :: mol_nxt(:)
       type(type_property), intent(inout):: property(:)
     end subroutine mingibbs
     
  end interface
end module interface_mingibbs





recursive subroutine mingibbs(temp, pres, emol, struct, property, mol_nxt, mol_ini)
  use prm_type, only: type_property
  use prm_common, only: min, minfr, cnv, zero, debug
  use ssl2_linear
  use interface_chk_emol
  use interface_chk_emolfr  
  use interface_chk_equiv
  use interface_chk_mu
  use interface_chk_converge
  use interface_chk_mol
  use interface_matrix_coeff
  use interface_matrix_const
  use interface_equivmol
  use interface_initmol
  use interface_gibbs_energy
  use interface_packing
  use interface_unpacking
  use interface_mingibbs
  use interface_display
  use interface_mol2emol
  use interface_def_phase
  use interface_phase_count

  implicit none
  real(8), intent(in)               :: temp  
  real(8), intent(in)               :: pres
  real(8), intent(in)               :: emol(:)
  real(8), intent(in)               :: struct(:,:)
  real(8), intent(in), optional     :: mol_ini(:)
  real(8), intent(out)              :: mol_nxt(:)
  type(type_property), intent(inout):: property(:)

  real(8)                           :: gibbs_pre
  real(8)                           :: gibbs_nxt
  real(8), allocatable              :: mol(:)
  real(8), allocatable              :: mol_pre(:)
  real(8), allocatable              :: mu(:)
  real(8), allocatable              :: mu0(:)
!  real(8), allocatable              :: A(:,:)
  real(8), allocatable              :: A(:)
  real(8), allocatable              :: B(:)
  real(8), allocatable              :: emol_np(:)
  real(8), allocatable              :: struct_np(:,:)
  real(8), allocatable              :: mol_ini_np(:)
  integer                           :: emax
  integer                           :: pmax
  integer                           :: i
  logical, allocatable              :: ssw(:)
  logical, allocatable              :: esw(:)
  logical                           :: sw
  logical                           :: cnvsw
  type(type_property), allocatable  :: property_np(:)
  
  !ɽ
  if (debug) then     
     write(*,*) ""
     write(*,*) "call mingibbs", temp, pres
     write(*,*) ""
     write(*,*) property%name
  end if

  !ֹդľ
  call def_phase(property)
  if (debug) write(*,*) "phase", property%phase
  
  !ͤ⾯ʤʤäǤ
  call chk_emol(emol, struct, min, ssw, esw, sw)
  if (sw) then 
     call rcv_mingibbs
     return
  end if
  
  !Ϥ줿ǿ򸵤˽
  if (present(mol_ini)) then 
     call initmol(emol, struct, mol, mol_ini)  
  else
     call initmol(emol, struct, mol)  
  end if
  
  !ǤΥ֥ͳͥ륮
  call gibbs_energy(temp, pres, mol, property, &
       & mu = mu, mu0 = mu0, gibbs = gibbs_pre)
  if (debug) call display(mol, "")

  !Ž꤬ñ¸ߤƤʤȤå
  call chk_equiv(mu0, property%phase, property%name, ssw, sw)
  if (sw) then
     call rcv_mingibbs
     return
  end if
  
  !Ǿ롼¹ν
  emax = size(struct, 2)
  call phase_count(property%phase, pmax)
  i = emax + pmax
!  call ssl2_lax_init(i)
  call ssl2_lsx_init(i)
!  allocate(A(i, i), B(i))
  allocate(A(i*(i+1)/2), B(i))

  !Ǿ롼
  min_gibbs: do 
     
     !
     call matrix_coeff(mol, property%phase, struct, A)	
     if (debug) call display(A, "")
     
     !
     call matrix_const(temp, mol, mu, property%phase, struct, B)
     if (debug) call display(B, "")
    
     !ϢΩ
!     call ssl2_lax(A, B)
     call ssl2_lsx(A, B)
!     call display(B, "")
     
     !ϢΩβ򤫤׻
     call equivmol(B, struct, property%phase, temp, mu, mol, mol_pre)
     call chk_mol(mol, mol_pre)
     if (debug)  call display(mol, "ؼ¸")
     
     !ʿǤΥ֥ͳͥ륮
     deallocate(mu)
     call gibbs_energy(temp, pres, mol, property, mu = mu, gibbs = gibbs_nxt)
     
     !ʿξ֥å
     call mol2emol(mol, struct, emol_np)
!     call chk_mu(mol, mu, minfr, ssw, sw)
     call chk_emolfr(emol_np, mol, struct, minfr, ssw, esw, sw)
     deallocate(emol_np)
     if (sw) then 
        call rcv_mingibbs
        return
     end if     
     
     !«å
!     call chk_converge(gibbs_pre, gibbs_nxt, cnv, cnvsw)
     call chk_converge(mol_pre, mol, cnv, cnvsw)
     if (debug) write(*,*) "gibbs check: ", gibbs_pre, gibbs_nxt, cnvsw
     if (cnvsw) then 
        call unpacking(mol, property%num, zero, mol_nxt)
        return 
     end if
     
!     if (gibbs_pre < gibbs_nxt) then 
!        call unpacking(mol, property%num, zero, mol_nxt)
!        write(*,*) "MINGIBBS: gibbs_free_energy is increase"
!        write(*,*) "temp: ", temp
!        write(*,*) "pres: ", pres
!     end if
     
     !롼פ󤹾
     gibbs_pre = gibbs_nxt
     
  end do min_gibbs
  
contains

  subroutine rcv_mingibbs
    real(8), allocatable :: struct_tmp(:,:)
    
    if (allocated(A)) deallocate(A, B)

    if (.not. allocated(esw)) then 
       allocate(esw(emax)); esw = .true. 
       call packing(struct, ssw, esw, struct_tmp)
       esw = sum(struct_tmp,1) /= 0.0d0
    end if

    call packing(emol, esw, emol_np)
    call packing(struct, ssw, esw, struct_np)
    call packing(property, ssw, property_np)

    if (present(mol_ini)) then 
       call packing(mol_ini, ssw, mol_ini_np)
       call mingibbs(temp, pres, emol_np, &
            & struct_np, property_np, mol_nxt, mol_ini_np)
    else    
       call mingibbs(temp, pres, emol_np, &
            & struct_np, property_np, mol_nxt)
    end if
  end subroutine rcv_mingibbs
  
  
end subroutine mingibbs
